ad_library {

    Data Aggregator

    @creation-date 2009-03-27
    @author Alessandro Landim <alessandro.landim@gmail.com>

}

namespace eval da {}
namespace eval da::aggregator {}
namespace eval da::field {}
namespace eval da::data {}


ad_proc -public da::import_xml {
    {-data_id:required}
	{-url:required}
} {
    Import XML
} {
		
	    if {[catch {set xml [ns_httpget $url 30 0]} result]} {
	   		#ERROR
	    	set xml ""
    	} 
	
		set xml [encoding convertfrom utf-8 $xml]

		set doc [dom parse $xml]
		if [catch {set root [$doc documentElement]} err] {
		       error "Error parsing XML: $err"
		}

		set nodes_main  [$root childNodes]

		set row 1
		foreach node_main $nodes_main {
			set nodes  [$node_main childNodes] 
	
			foreach node $nodes {
		         set node_name [$node nodeName]
				
				 if {![info exists fields_$node_name]} {
					 if [catch {set pretty_name [$node getAttribute "pretty_name"]} err] {
							set pretty_name ""
			         }
				 
				 	 set field_id [da::field::new -data_id $data_id -key $node_name -pretty_name $pretty_name]
					 set fields_$node_name $field_id
				 }

		         set out [$node text]
				  
				 da::data::new -field_id [value_if_exists fields_$node_name] -sort_order $row -data $out

			}
			incr row
		}
		
		return $data_id

}	


ad_proc -public da::aggregator::info {
    {-data_id:required}
	{-array:required}
} {
    Get Aggregator information 
} {
	upvar $array row
	db_1row get_aggregator_info {select name, description, url from data_aggregators where data_id = :data_id} -column_array row
	
}

ad_proc -public da::field::get_list {
    {-data_id:required}
} {
    Get a list of fields from data_id. List with id and key.
} {
	db_multirow get_fields select_fields {
			select key, field_id
			from data_aggregators_fields
			where data_id = :data_id
	} {
			lappend result [list $field_id $key]

	}
	return $result
}


ad_proc -public da::update_xml {
    {-data_id:required}
} {
    Update XML
} {
	
	db_dml delete_data_all {
				delete from data_aggregators_datas where field_id in (select field_id from data_aggregators_fields where data_id = :data_id)
	}

	da::aggregator::info -data_id $data_id -array data
	
	if {[catch {set xml [ns_httpget $data(url) 30 0]} result]} {
	   		#ERROR
	    	set xml ""
    } 
	
	#set xml [exec cat /tmp/rss.xml]
	set xml [encoding convertfrom utf-8 $xml]
	set doc [dom parse $xml]

	if [catch {set root [$doc documentElement]} err] {
		       error "Error parsing XML: $err"
	}

	set fields_list [da::field::get_list -data_id $data_id]

	foreach field $fields_list {
			set key [lindex $field 1]
			set field_id [lindex $field 0]

			set fields_$key $field_id
	}

	set nodes_main  [$root childNodes]

	set row 1
	foreach node_main $nodes_main {
		set nodes  [$node_main childNodes]
	
		foreach node $nodes {
	         set node_name [$node nodeName]

			
				
			 if {![info exists fields_$node_name]} {
					 if [catch {set pretty_name [$node getAttribute "pretty_name"]} err] {
							set pretty_name ""
			         }
				 
				 	 set field_id [da::field::new -data_id $data_id -key $node_name -pretty_name $pretty_name]
					 set fields_$node_name $field_id
			 } elseif ![catch {set pretty_name [$node getAttribute "pretty_name"]} err] {
					da::field::update_pretty_name -field_id [value_if_exists fields_$node_name] -pretty_name $pretty_name
			 }
				

	         set out [$node text]
				  
			 da::data::new -field_id [value_if_exists fields_$node_name] -sort_order $row -data $out
		}
		incr row
	}

	set fields_list [db_list get_fields {select field_id from data_aggregators_fields where data_id = :data_id}]

	foreach field_id $fields_list {

			## check if field_id is in use
			set check [db_list check {select 1 from data_aggregators_datas where field_id = :field_id}]

			if {$check == ""} {
					da::field::delete -field_id $field_id
			}

	}
		
	return $data_id


}


ad_proc -public da::aggregator::new {
	{-name:required}
	{-description:required}
	{-url:required}
} {
    create new aggregator
} {
		set package_id 	  [ad_conn package_id]
		set creation_user [ad_conn user_id]
		set creation_ip 	  [ad_conn peeraddr]
		
		set data_id [db_string create_aggregator {
			select data_aggregator__new (
				null, 			-- data_id
			    :name,			-- name
			    :description,	-- description
				:url,			-- url
			    :package_id,	-- package_id
			    now(), 			-- creation_date
			    :creation_user, -- creation_user
			    :creation_ip, 		-- creation_ip
			    null			-- context_id
			)
		}]

		da::import_xml -data_id $data_id -url $url
		
		return $data_id

}

ad_proc -public da::field::delete {
	{-field_id:required}
} {
    delete field 
} {
		db_exec_plsql delete_field {
			select data_aggregator_field__del (:field_id)
		}
}

ad_proc -public da::field::update_pretty_name {
	{-field_id:required}
	{-pretty_name:required}
} {
    update field pretty_name
} {
		db_dml update_pretty_name {
			update data_aggregators_fields set pretty_name = :pretty_name where field_id = :field_id
		}
}


ad_proc -public da::field::new {
    {-data_id:required}
	{-key:required}
	{-pretty_name ""}
} {
    create new field
} {

	set package_id 	  [ad_conn package_id]
	set creation_user [ad_conn user_id]
	set creation_ip 	  [ad_conn peeraddr]
		
	set field_id [db_string create_field {
		select data_aggregator_field__new (
				null, 			-- field_id
			    :data_id,		-- data_id
			    :key, 			-- key
				:pretty_name, 	-- pretty_name
			    :package_id,	-- package_id
			    now(), 			-- creation_date
			    :creation_user, -- creation_user
			    :creation_ip, 		-- creation_ip
			    null			-- context_id
		)
	}]

	return $field_id
}

ad_proc -public da::data::new {
    {-field_id:required}
	{-sort_order:required}
	{-data:required}
} {
    create new data
} {

		
	db_exec_plsql create_data {
		select data_aggregator_data__new (
				:field_id,		-- field_id
			    :sort_order,	-- sort_order
			    :data 			-- data
		)
	}


}




