#  This file is part of dotLRN.
#  dotLRN is free software; you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation; either version 2 of the License, or (at your option) any later
#  version.
#  dotLRN is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.

ad_library {
    
    the dotlrn applet for data aggregator
    
    @author alessandro.landim@gmail.com
}

namespace eval dotlrn_data_aggregator {}

ad_proc -public dotlrn_data_aggregator::package_key {
} {
    What package does this applet deal with?
} {
    return "data-aggregator"
}

ad_proc -public dotlrn_data_aggregator::my_package_key {
} {
    What's my package key?
} {
    return "dotlrn-data-aggregator"
}

ad_proc -public dotlrn_data_aggregator::applet_key {
} {
    What's my applet key?
} {
    return "dotlrn_data_aggregator"
}

ad_proc -public dotlrn_data_aggregator::get_pretty_name {
} {
} {
    return "Data Aggregator"
}

ad_proc -public dotlrn_data_aggregator::add_applet {
} {
    Called for one time init - must be repeatable!
    @return new pkg_id or 0 on failure
} {
    
    # FIXME: won't work with multiple dotlrn instances Use the package_key
    # for the -url param - "/" are not allowed!
    
    set package_id 0
    
    if {![dotlrn::is_package_mounted \
	      -package_key [package_key]]} {
		set package_id [dotlrn::mount_package \
			    -package_key [package_key] \
			    -url [package_key] \
			    -directory_p "t"]
    }
    
    dotlrn_applet::add_applet_to_dotlrn -applet_key [applet_key] -package_key [my_package_key]
}

ad_proc -public dotlrn_data_aggregator::remove_applet {
} {
    One-time destroy for when the entire applet is removed from dotlrn.
} {
    ad_return_complaint 1 "[applet_key] remove_applet not implimented!"
}

ad_proc -public dotlrn_data_aggregator::data_aggregator_create_helper {
    {-community_id:required}
    {-package_id:required}
} {
    A helper proc to create a calendar for a comm, returns the new calendar_id
} {
    return "0"
}

ad_proc -public dotlrn_data_aggregator::add_applet_to_community {
    community_id
} {
    Add the calendar applet to a specific dotlrn community
} {
    set results [add_applet_to_community_helper \
		     -community_id $community_id
		]
    return [lindex $results 0]
}

ad_proc -public dotlrn_data_aggregator::add_applet_to_community_helper {
    {-community_id:required}
} {
    Add the Maps applet to a specific dotlrn community
    
    @params community_id
} {
    
    # Create and Mount the Maps Package

    set package_id [dotlrn::instantiate_and_mount \
			-mount_point "data-aggregator" \
			$community_id \
			[package_key] \
		       ]
    
    # Set up the project manager portlet for this portal/community
    
    set portal_id [dotlrn_community::get_portal_id \
		       -community_id $community_id \
		      ]
 
   set community_name [dotlrn_community::get_community_name $community_id]
   

    # Add all portlets to the Portal.
    
    data_aggregator_portlet::add_self_to_page \
	-portal_id $portal_id \
	-package_id $package_id \
	-community_id $community_id

    # this should return the package_id
    return $package_id
}

ad_proc -public dotlrn_data_aggregator::remove_applet_from_community {
    community_id
} {
    remove the applet from the community
} {
    ad_return_complaint 1 "[applet_key] remove_applet_from_community not implimented!"
}

ad_proc -public dotlrn_data_aggregator::add_user {
    user_id
} {
    Called once when a user is added as a dotlrn user.
    Create a private, personal, global Project Manager for the User if they don't have one and add it to the user's portal
} {
}

ad_proc -public dotlrn_data_aggregator::remove_user {
    user_id
} {
    Remove a user from dotlrn
} {
    
    # Not yet implemented.
}

ad_proc -public dotlrn_data_aggregator::add_user_to_community {
    community_id
    user_id
} {
    Add a user to a community
} {
    
    set package_id [dotlrn_community::get_applet_package_id -community_id $community_id -applet_key [applet_key]]
    set portal_id [dotlrn::get_portal_id -user_id $user_id]
    # use "append" here since we want to aggregate
    set param_action append
    
    data_aggregator_portlet::add_self_to_page \
        -portal_id $portal_id \
        -package_id $package_id \
	-community_id $community_id \
        -param_action $param_action
    
}

ad_proc -public dotlrn_data_aggregator::remove_user_from_community {
    community_id
    user_id
} {
    Remove a user from a community
} {

    set package_id [dotlrn_community::get_applet_package_id -community_id $community_id -applet_key [applet_key]]
    set portal_id [dotlrn::get_portal_id -user_id $user_id]
    
    # use "append" here since we want to aggregate
    set param_action append
   
    #remove  portlet
    data_aggregator_portlet::remove_self_from_page \
          -portal_id $portal_id \
	  -community_id $community_id

}

ad_proc -public dotlrn_data_aggregator::add_portlet {
    portal_id
} {
    Set up default params for templates about to call add_portlet_helper
    
    @param portal_id
} {

    data_aggregator_portlet::add_self_to_page \
	-portal_id $portal_id \
	-community_id 0 \
	-package_id 0

    
}

#ad_proc -private dotlrn_data_aggregator::add_portlet_helper {
#aportal_id # args #} {# Does the call to add the portlet to the
#aportal.  # Params for the portlet are sent to this proc by the
#acaller.  #} {# calendar_portlet::add_self_to_page \
    -portal_id $portal_id \
    -pretty_name [ns_set get $args "pretty_name"] \
    -calendar_id [ns_set get $args "calendar_id"]  \
    -scoped_p [ns_set get $args "scoped_p"] \
    -param_action [ns_set get $args "param_action"]

#    calendar_full_portlet::add_self_to_page \
    -portal_id $portal_id \
    -page_name [ns_set get $args "full_portlet_page_name"] \
    -calendar_id [ns_set get $args "calendar_id"]  \
    -scoped_p [ns_set get $args "scoped_p"] \
    -param_action [ns_set get $args "param_action"]

#}

ad_proc -public dotlrn_data_aggregator::remove_portlet {
    portal_id
    args
} {
    A helper proc to remove the underlying portlet from the given portal.
    This is alot simpler than add_portlet.
    
    @param portal_id
    @param args An ns_set with the data_id.
} {
    data_aggregator_portlet::remove_self_from_page \
	-portal_id $portal_id \
	-package_id [ns_set get $args "package_id"]

}

ad_proc -public dotlrn_data_aggregator::clone {
    old_community_id
    new_community_id
} {
    Clone this applet's content from the old community to the new one
} {
}

ad_proc -public dotlrn_data_aggregator::change_event_handler {
    community_id
    event
    old_value
    new_value
} {
    listens for the following events: rename
} {
    switch $event {
	rename {
	    
	    #	    handle_rename -community_id $community_id -old_value
	    #	$old_value -new_value $new_value
	}
    }
}

ad_proc -private dotlrn_data_aggregator::handle_rename {
    {-community_id:required}
    {-old_value:required}
    {-new_value:required}
} {
    what to do in calendar when a dotlrn community is renamed
} {
    
    #    calendar::rename -calendar_id [get_group_calendar_id
    # -community_id $community_id] -calendar_name $new_value
}

# Some dotlrn_data_aggregator specific procs
